I'm sorry u/{{ message.author }}, you don't have any pending tips. Perhaps they have already been confirmed, declined, or expired. Tips are auto-confirmed after you've registered.

{% include 'footer.tpl' %}
